#!/usr/bin/python
# coding=utf-8

# This file is part of fractgtk
# Copyright (C) 2007 - Dennis Schulmeister
#
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# It is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this file. If not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor,
# Boston, MA  02110-1301  USA

'''
    This module contains the base class for all fractal types.
'''

# Python modules
import os.path
import gtk.gdk

class Fractal:
    '''
        This is the base class for all available fractal types.
    '''
    
    def __init__(self, app):
        '''
            Constructor. Sets default values and default palette.
        '''
        self.app = app
        self.defaultTopLeft = 0 + 0j
        self.defaultBottomRight = 1 + 1j
        self.bail = 4
        self.maxIter = 50
        
        self.palette = []
        # self.prepare_palette()

    def parameter_dialog(self):
        '''
            This method is meant to be overridden by anchestors. Its purpose is to
            present a dialog to the user where specific parameters can be entered.
        '''
        pass

    def prepare_palette(self, drawable):
        '''
            This method builds a default palette.
        '''
        paletteFile = os.path.join(self.app.appPath, "palette.png")
        bitmap = gtk.gdk.pixbuf_new_from_file(paletteFile)
        width  = bitmap.get_width()
        height = bitmap.get_height()
        
        drawable = gtk.gdk.Pixmap(gtk.gdk.get_default_root_window(), width, height, -1)
        gc = drawable.new_gc()
        drawable.draw_pixbuf(gc, bitmap, 0, 0, 0,0 , width, height, gtk.gdk.RGB_DITHER_NORMAL, 0, 0)
        image = drawable.get_image(0, 0, width, height)
        
        self.palette = []
        for nr in range(self.maxIter + 1):
            x = (width / (self.maxIter + 1.0)) * nr
            x = int(x)
            hColor = image.get_pixel(x, 1)
            
            color = gtk.gdk.Color(pixel=hColor)
            gc = gtk.gdk.GC(drawable, color, color)
            self.palette.append(gc)

    def draw(self, window, progress, topLeft, bottomRight):
        '''
            Children need to overwrite the method. It renders the fractal to the
            given gtk.gdk.window.
        '''
        pass
