#!/usr/bin/python
# coding=utf-8

# This file is part of fractgtk
# Copyright (C) 2007 - Dennis Schulmeister
#
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# It is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this file. If not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor,
# Boston, MA  02110-1301  USA

'''
    This dialog lets the user choose from several available fractal types.
'''

# Python modules
import pygtk
import gtk
import gtk.glade
import gobject
pygtk.require('2.0')

class FractalType:
    '''
        This dialog lets the user choose one of several fractal types.
    '''

    def __init__(self, app=None, dict={}):
        '''
            Constructor.
        '''
        if (not app):
            raise "Fractal Dialog needs the main application instance."
        if (not dict):
            raise "Fractal Dialog needs a list of all available fractal types."
        
        # Store instance variables
        self.app = app
        self.dict = dict
        
        self.gladeFile = self.app.gladeFile
        self.wTree     = gtk.glade.XML(self.gladeFile, root="dlgFractType")
        self.dlg       = self.wTree.get_widget("dlgFractType")
        self.tree      = self.wTree.get_widget("treeFractList")
        self.btnOk     = self.wTree.get_widget("btnFractTypeOk")
        self.btnCanc   = self.wTree.get_widget("btnFractTypeCanc")
        
        self.dlg.connect("delete-event", self.dlg.hide_on_delete)
        self.dlg.set_icon_from_file(self.app.iconFile)
        
        # Initialize list content
        self.listStore = gtk.ListStore(gobject.TYPE_STRING)
        for fractal in self.dict:
            self.listStore.append([fractal])
        
        self.tree.set_model(self.listStore)
        cell1 = gtk.CellRendererText()
        col1 = gtk.TreeViewColumn("Available fractals", cell1, text=0)
        col1.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        self.tree.append_column(col1)

    def run(self):
        '''
            This method runs the dialog in a modal state. If the user selects a
            fractal a new fractal object will be returned. Otherwise None will
            be returned.
        '''
        response = self.dlg.run()
        self.dlg.hide()
        
        if (response < 0):
            return None
        
        sel = self.tree.get_selection()
        (model, iter) = sel.get_selected()
        path = self.listStore.get_path(iter)
        row  = self.listStore[path]
        name = row[0]
        
        return self.dict[name](self.app)
