#!/usr/bin/python
# coding=utf-8

# This file is part of fractgtk
# Copyright (C) 2007 - Dennis Schulmeister
#
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# It is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this file. If not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor,
# Boston, MA  02110-1301  USA

'''
    This is image resolution chooser. It lets the user specify a resolution
    for an image to be rendered.
'''

# Python modules
import pygtk
import gtk
import gtk.glade
import gobject
pygtk.require('2.0')

class ImageResolution:
    '''
        This dialog lets the user specify a resolution for an image to
        be rendered.
    '''

    def __init__(self, app=None):
        '''
            Constructor.
        '''
        if (not app):
            raise "Image Resolution Dialog needs the main application instance."
        
        # Store instance variables
        self.app = app
        
        self.gladeFile = self.app.gladeFile
        self.wTree     = gtk.glade.XML(self.gladeFile, root="dlgImageResolution")
        self.dlg       = self.wTree.get_widget("dlgImageResolution")
        self.entXReso  = self.wTree.get_widget("entXReso")
        self.entYReso  = self.wTree.get_widget("entYReso")
        self.btnFit    = self.wTree.get_widget("btnResoFit")
        
        self.btnFit.connect("clicked", self.adjust_ratio)
        
        self.dlg.connect("delete-event", self.dlg.hide_on_delete)
        self.dlg.set_icon_from_file(self.app.iconFile)

    def run(self):
        '''
            This method runs the dialog in a modal state. If the user enters a
            image resolution it will be returned as a tupple: (width, height).
            Otherwise None will be returned.
        '''
        # Prefill entry fields
        try:
            xReso = int(self.entXReso.get_text())
        except:
            xReso = 0
        
        try:
            yReso = int(self.entYReso.get_text())
        except:
            yReso = 0
        
        try:
            if not xReso:
                xReso = self.app.wndMain.pixmap.get_width()
            if not yReso:
                yReso = self.app.wndMain.pixmap.get_height()
            
            self.entXReso.set_text(str(xReso))
            self.entYReso.set_text(str(yReso))
        except:
            pass
        
        # Present dialog
        response = self.dlg.run()
        self.dlg.hide()
        
        if (response < 0):
            return None
        
        # Process user input
        try:
            xReso = int(self.entXReso.get_text())
        except:
            xReso = 0
        
        try:
            yReso = int(self.entYReso.get_text())
        except:
            yReso = 0
        
        return (xReso, yReso)

    def adjust_ratio(self, widget=None, data=None):
        '''
            This method automatically adjusts the image resolutio so that its
            aspect ratio fits the aspect ratio of the fractal to be calculated.
        '''
        # Calculate fractal aspect ratio
        firstCorner  = self.app.wndMain.topLeft
        secondCorner = self.app.wndMain.bottomRight
        
        try:
            fractalWidth  = abs(secondCorner.real - firstCorner.real) / 1.0
            fractalHeight = abs(secondCorner.imag - firstCorner.imag) / 1.0
            fractalRatio  = fractalWidth / fractalHeight
        except:
            return
        
        # Calculate current image ratio
        try:
            xReso = int(self.entXReso.get_text())
            yReso = int(self.entYReso.get_text())
        except:
            xReso = 800
            yReso = 600
        
        imageRatio = (xReso / 1.0) / (yReso / 1.0)
        
        # Correct image size according to fractal ratio
        if (imageRatio == fractalRatio):
            return
        
        if (xReso > yReso):
            xReso = (fractalWidth / fractalHeight) * yReso
        else:
            yReso = (fractalHeight / fractalWidth) * xReso
        
        # Return calculated values
        xReso = int(xReso)
        yReso = int(yReso)
        
        self.entXReso.set_text(str(xReso))
        self.entYReso.set_text(str(yReso))
